/*
 **************************************************************************
 *		SWI-Prolog ActiveX Extensions.
 *
 *	This software is provided in source and binary form free of charge to 
 *  users of SWI-Prolog.  No warranty of any kind is expressed or implied;
 *  in particular, this is entirely unsupported software.
 *
 *  Documentation for this product is available from the SWI-Prolog 
 *  web site (http://www.swi.psy.uva.nl/projects/SWI-Prolog/).
 *
 *	Version 1.0.1.
 *
 *	October 1, 1999
 *
 **************************************************************************
 */

//
//	ZSTR.H:	String management
//

#ifndef _ZSTR_H_
#define _ZSTR_H_

#include <stdarg.h>		//  For "va_list"
#include <tchar.h>		//  For TCHAR
#include <malloc.h>		//  For "alloca"
#include <string>		//  STL string class

#include "basics.h"

////////////////////////////////////////////////////////////////////
//	class ZSTR
////////////////////////////////////////////////////////////////////
template <class _TCH>
class T_ZSTR : public basic_string<_TCH, char_traits<_TCH> >
{
	typedef basic_string<_TCH, char_traits<_TCH>, allocator<_TCH> > t_string;
	typedef _TCH * _TSZ;
	typedef const _TCH * _TSZC;

  public:
	T_ZSTR ( const char * pach )	{ self = pach;		}
	T_ZSTR ( const wchar_t * pwch ) { self = pwch;		}
	T_ZSTR ()						{ self = _T("");	}

	_TSZC Tszc() const				{ return c_str();	}
	inline operator _TSZC () const	{ return Tszc();	}
	void Reset ()					{ resize(0);		}

	T_ZSTR & operator = ( const wchar_t  * pwch )
	{
		if ( pwch ) 
			_t_assign( pwch, self );
		else
			Reset();
		return *this;
	}
	T_ZSTR & operator = ( const char * pach )
	{
		if ( pach ) 
			_t_assign( pach, self );
		else
			Reset();
		return *this;
	}

	template <class _CH>
	T_ZSTR & operator = ( const T_ZSTR<_CH> & tzstr )
	{
		_t_assign( tzstr.Tszc(), self );
		return *this;
	}

	int FormatAppend ( _TSZC tszcFmt, ... )
	{
		T_ZSTR strTemp ;
		va_list	valist;
		va_start( valist, tszcFmt );
		
		int ccb = strTemp.Vsprintf( tszcFmt, valist ) ;
		va_end( valist );
		
		self += strTemp;
		return ccb;  // Return Vsprintf() result
	}

	int Format ( _TSZC tszcFmt, ... )
	{
		va_list	valist;
		va_start( valist, tszcFmt );
		
		int ccb = Vsprintf( tszcFmt, valist ) ;
		
		va_end( valist );
		return ccb;  // Return Vsprintf() result
	}
	int Vsprintf ( _TSZC tszcFmt, va_list valist )
	{
		return _t_VsprintZstr( self, tszcFmt, valist );
	}
	void ToLower ();
	void ToUpper ();
};

//  Helper functions in ZSTR.CPP
extern int _t_vsntprintf(char * pch, size_t cch, const char * pszFormat, va_list argList);
extern int _t_vsntprintf(wchar_t * pch, size_t cch, const wchar_t * pszFormat, va_list argList);
extern void _t_assign ( const char * pasz, T_ZSTR<char> & zstr );
extern void _t_assign ( const wchar_t * pwsz, T_ZSTR<wchar_t> & zstr );
extern void _t_assign ( const wchar_t * pwsz, T_ZSTR<char> & zstr );
extern void _t_assign ( const char * pasz, T_ZSTR<wchar_t> & zstr );
extern int _t_VsprintZstr ( T_ZSTR<char> & zs, const char * tszcFmt, va_list valist );
extern int _t_VsprintZstr ( T_ZSTR<wchar_t> & zs, const wchar_t * tszcFmt, va_list valist );
//  Maximum size allowed for "Vsprintf" formatting; formatting beyond this size will fail,
//		but you may change this value globally
extern int g_cbVsprintfMaxBufferSize;	//  = 1000000 (1MB)

//
//	Standardly used types
//
typedef T_ZSTR<TCHAR> ZSTR;			//  default string class
typedef T_ZSTR<char> ZSTRA;			//  "narrow" 8-bit ANSI/MBCS string
typedef T_ZSTR<wchar_t> ZSTRW;		//  "wide" 16-bit Unicode string
	

//  String array class declarations
DEFINEV(ZSTR);
DEFINEV(VZSTR);
DEFINEV(ZSTRA);
DEFINEV(VZSTRA);
DEFINEV(ZSTRW);
DEFINEV(VZSTRW);

// end of ZSTR.H

#endif
